library persistent_storage;

import 'dart:html';
import 'package:intl/intl.dart';

class PersistentData {
  /// this should be called at the beginning of each page
  static void load() {
    if (_loaded) return;
    try {
      if (window.sessionStorage == null)
        throw new Exception('sessionStorage disabled');
      if (window.localStorage == null)
        throw new Exception('localStorage disabled');

      window.sessionStorage.forEach((key, value) => _sessionData[key] = value);
      window.localStorage.forEach((key, value) => _localData[key] = value);

      /// this will trigger a DOMException 22 in iOS private mode, triggering cookies
      window.sessionStorage['test'] = 'test';
      window.sessionStorage.remove('test');
      _cookieMode = false;
      _loaded = true;
    } on Exception {
      if (!window.navigator.cookieEnabled)
        throw new Exception(
            'This page requires cookies to function properly, please enable it in your browser or contact us if the problem persists.');
      // make sure cookies actually work
      final timeNow = new DateTime.now().toUtc();
      final localExpireTime = timeNow.add(new Duration(days: 365));
      final formatter = new DateFormat('E, dd LLL yyyy HH:mm:ss');
      window.document.cookie =
          'test=cookie;expires=${formatter.format(localExpireTime)} UTC;';
      if (!window.document.cookie.contains('test')) {
        throw new Exception(
            'This page requires cookies to function properly, please enable it in your browser or contact us if the problem persists.');
      }

      window.document.cookie.split('; ')
        ..forEach((pair) {
          final keyValue = pair.split('=');
          _localData[keyValue.first] = keyValue.last;
        });
      _cookieMode = true;
      _loaded = true;
    }
  }

  static void _save() {
    if (_loaded == false)
      throw new StateError('error_persistent_data_not_loaded');
    if (_cookieMode == true) {
      // Delete all previous cookies
      window.document.cookie.split('; ')
        ..forEach((pair) {
          final pairs = pair.split('=');
          window.document.cookie =
              '${pairs.first}=;expires=Thu, 01 Jan 1970 00:00:00 GMT;path=/';
        });

      /// Have the cookie to expire in two hours from now
      final timeNow = new DateTime.now().toUtc();
      final localExpireTime = timeNow.add(new Duration(days: 365));
      final formatter = new DateFormat('E, dd LLL yyyy HH:mm:ss');

      _localData.forEach((key, value) {
        window.document.cookie =
            '$key=$value;expires=${formatter.format(localExpireTime)} UTC;path=/';
      });
    } else {
      window.sessionStorage.clear();
      _sessionData.forEach((key, value) => window.sessionStorage[key] = value);
      window.localStorage.clear();
      _localData.forEach((key, value) => window.localStorage[key] = value);
    }
  }

  static void set(String key, String value, {bool permanent: false}) {
    if (_loaded == false)
      throw new StateError('error_persistent_data_not_loaded');
    if (permanent == true || cookieMode == true)
      _localData[key] = value;
    else
      _sessionData[key] = value;
    _save();
  }

  static void clear({bool clear_permanent: true}) {
    _sessionData.clear();
    if (clear_permanent == true || cookieMode == true) _localData.clear();
    _save();
  }

  static String get(String key, {bool permanent: false}) {
    if (_loaded == false) throw new StateError('persistent_storage_not_loaded');

    if (permanent == true || cookieMode == true)
      return _localData.containsKey(key) ? _localData[key] : '';
    else
      return _sessionData.containsKey(key) ? _sessionData[key] : '';
  }

  static bool get cookieMode => _cookieMode;

  static bool contains(String key, {bool permanent: false}) {
    if (_loaded == false) throw new StateError('persistent_storage_not_loaded');
    return (permanent == true || cookieMode == true)
        ? _localData.containsKey(key)
        : _sessionData.containsKey(key);
  }

  static void remove(String key, {bool permanent: false}) {
    if (permanent == true || cookieMode == true)
      _localData.remove(key);
    else
      _sessionData.remove(key);
    _save();
  }

  static List<String> get localKeys => _localData.keys.toList(growable: false);
  static List<String> get sessionKeys =>
      _sessionData.keys.toList(growable: false);
  static bool get loaded => _loaded;

  static bool _loaded = false;
  static bool _cookieMode = false;

  static final Map<String, String> _sessionData = {};
  static final Map<String, String> _localData = {};
}
